// Save chrome, return null image file via CGI "$Revision: 1.2 $"
//
// Based http://cscene.org/CS6/CS6-07.html 14 Dec 2001
// W. B. Langdon

// make using g++ -o harvest2 harvest2.cc #eg on occasion

// 21 Dec 2001

//Modifications (in reverse order)

// WBL  8 Sep 2004 For use with fileUpload
//                 Add check for duplicate chromes
//                 check_header() read() based on seed.cc r1.4
// WBL 18 Mar 2004 Make happier to www users. Rename dump www_header
// WBL  8 Jan 2004 Make more robust against spiders.
//                 Log errors to debug and back to internet invoker

// WBL 29 Dec 2001 append to live population


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <iostream.h>
#include <fstream.h>
#include <time.h>
#include <hash_map>

#define strcasecmp strcmp

#if _WIN32
#include <fcntl.h>
#include <io.h>

#define strcasecmp stricmp
#endif

#define debug_file "debug"

void www_header()
{
  //cout << "Content-Type: " << "image/gif" << endl;
  //cout << endl;
  //cout.write(buf, count);
  cout << "Content-Type: text/plain\n\n";
}

int hexdigit(char c) {
  return 
    (c>='A' && c<='F')? c - 'A'+10 :
    (c>='a' && c<='f')? c - 'a'+10 :
    (c>='0' && c<='9')? c - '0'    :  -1000; //error
}

void header(ostream& out) {
  time_t start;
  time (&start);
  char* timestr = ctime(&start); timestr[strlen(timestr)-1]='\0';
  memcpy(timestr,"Fil",3); //indicate harvest2 rather than harvest
                          //cf check_header
  out << timestr << " ";
  out << getenv("REMOTE_HOST") << " ";
  out << getenv("REMOTE_ADDR") << "|"; /* << flush;*/
}

void error_text(const char* text) {
  cout<<"\nERROR ";
  cout<<text<<endl; //send error message back to www user

  ofstream debug (debug_file,ios::app);
  debug<<"harvest2 failed ";
  header(debug);
  debug<<text<<endl;
  exit(0);
}

char* check_header(/*const*/ char* buf, int& size) {
  //const char* Alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  const char* alphabet = "abcdefghijklmnopqrstuvwxyz";
  //  const char* digits   = "0123456789";
  char* p;
  if(size>23 &&
     strspn(&buf[0],"SMTWF")==1 &&
     strspn(&buf[1],alphabet)==2 &&
     strspn(&buf[4],"JFMASOND")==1 &&
     strspn(&buf[5],alphabet)==2 &&
     strspn(&buf[7]," :0123456789")>15 &&
     (p = strchr(&buf[22],'|')) != NULL) {
    p++;
    size -= (p-buf);
    return p;
  }
  return buf;
}

int check_www_header(const char* p,const char* r, char** Q) {
  //ofstream debug (debug_file,ios::app);
  //debug<<r<<endl;
  //debug<<"["<<(p-r)<<","<<flush;
  if((p-r)<38)      return 0;
  if( strstr(r,"Sun,")!=r &&
      strstr(r,"Mon,")!=r &&
      strstr(r,"Tue,")!=r &&
      strstr(r,"Wed,")!=r &&
      strstr(r,"Thu,")!=r &&
      strstr(r,"Fri,")!=r &&
      strstr(r,"Sat,")!=r    ) return 0;
  char* q=strstr(r,"0.0.0.0|");
  //debug<<q<<","<<(q-r)<<","<<flush;
  if(q==NULL || (q-r)<30)      return 0;
  *Q = q+8;
  //debug<<"]\n";
  return 1;
}

vector<char*> www_seeds; 
int www_seed_count = 0;

void insert(const char* p, const char* q) {
  const int size = p-q;

  //ofstream debug (debug_file,ios::app);
  //debug<<"insert(";
  //for(int i=0;i<size;i++) debug<<q[i];
  //debug<<")"<<endl;   

  if(size<1) return;
  char* t = new char[size+1];
  memcpy(t,q,size*sizeof(char)); 
  t[size]='\0';
  www_seeds.insert(www_seeds.begin()+www_seed_count, t); 
  www_seed_count++;
}
int read_www() {
  cout<<"Reading from your file"<<flush;
  const int content_length=atoi(getenv("CONTENT_LENGTH"));
  cout<<"("<<content_length<<")"<<flush;
  int bytes=0;
  char buf[32768];
  char* p = buf;
  char* r = buf;
  do {
    int c=cin.get();
    bytes++;
    for(; c!='\n' && bytes<=content_length;) { 
      if(c==EOF) error_text("Ran into EOF on www input");
      if(c>=32&&c<=126) *p++ = char(c); //ignore control codes
      c = cin.get();
      bytes++;
    }
    p[0]='\0';
    char* q = buf;
    if(check_www_header(p,r,&q)) {
      insert(r,buf);
      const int t = p-q;
      memcpy(buf,q,t*sizeof(char));
      p=buf+t;
      r=p;
    }
    else {
      //This line does not start with valid header
      //reset if no previous valid text or empty line
      if(r==buf)      p=buf;
      else if(p==r) { insert(r,buf); r=p=buf; }
      else            r=p;
    }
  } while(bytes<=content_length);
  if(bytes==(content_length+1)) {
    bytes--;
    insert(p,buf);
  }
  else {
    char buf[80];
    sprintf(buf,"bytes read (%d) and content_length (%d) do not match.",
	         bytes,content_length);
    error_text(buf);
  }

  //ofstream debug (debug_file,ios::app);
  //debug<<"Read "<<bytes<<" bytes ";
  //debug<<www_seed_count<<" seeds "<<flush;
  cout <<", found "<<www_seed_count<<" seeds.\n";

  return (www_seed_count>0);
}

//http://www.sgi.com/tech/stl/hash_map.html
struct eqstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) == 0;
  }
};

hash_map<const char*, int, hash<const char*>, eqstr> seeds;
void read(fstream& seedf) {
    cout<<"Reading global population"<<flush;

    int seed_count = 0;
    char buf[32768];
    char c;
    do {
      char* p = buf;
      for(c=seedf.get(); c!='\n' && c!=EOF; p++) { 
	//debug<<" r"<<int(c)<<flush;
	*p = c; 
	c = seedf.get();
      }
      *p++='\0';
      //debug<<" read "<<int(c)<<"'"<<buf<<"'"<<flush;
      int size = p-buf;
      //debug<<size<<" "<<flush;
      if(size>1) {
	p = check_header(buf,size);
	char* t = new char[size];
	memcpy(t,p,size); 
	seed_count++;
	seeds[t] += 1;
	//cout<<"seeds["<<t<<"]\t"<<seeds[t]<<endl;
      }
    } while(c!=EOF);
    cout<<", "<<seed_count<<" seeds found."<<endl;
}

int main()
{
    //let remote end know we are here.
    www_header();

    //first get data from www (before it is lost)
    if(!read_www()) cout<<"ERROR no valid seeds in file\n";
    else {
      //Examples in http://www.cplusplus.com/doc/tutorial/tut6-1.html
      //suggest can both read and write file bt does not seem to work.
      fstream pop ("seed.pop",ios::in);
      if(!pop) error_text("failed to open seed.pop");
      else {
	read(pop);
	pop.close();
	pop.open("seed.pop",ios::app);
	int saved=0;
	for(int i=0;i<www_seed_count;i++) {
	  const char* imgparm=www_seeds[i];
	  if(seeds[imgparm]>0) {
	    cout<<"ERROR Seed '"<<imgparm<<"' already in global population\n";
	  }
	  else {
	    header(pop);
	    pop<<imgparm<<endl;
	    cout<<"Added       "<<imgparm<<endl;
	    saved++;
	  }
	}
	if(saved>0) cout<<"\n\nTHANK YOU\n"
			<<saved
			<<" flakes added to global population!\n";
	else        cout<<"\nNo flakes added:-(\n";
      }
    }

    return 0;
}
