#!/opt/ucl/bin/perl5

#WBL 18 Dec 2001 convert to white on black

#! /opt/ucl/bin/perl
#! /usr/local/bin/perl -w
# This script converts a homebrewed drawing format to .ppm format
#
# pl files contains
# P point-id x-coordinate y-coordinate
# C x-coordinate y-coordinate radius
# L start-point-id end-point-id
# # comment
# TR transform x-trans y-trans x-scale y-scale x-center y-center a-rotation
# empty lines are ignored.
sub round {
  my($x) = @_;
  return int($x+0.5) if $x >= 0;
  return int($x-0.5);
}
$before = "";
$output = "";
$after  = "";
$tx = $ty = 0.0;
$sx = $sy = 1.0;
$outx = $outy = 0.2;
$cx = $cy = 0.0;
$ang = 0.0;
$sin = sin $ang;
$cos = cos $ang;
$circ = 40; #circles will be drawn as polygons with circ edges
$input = "";
while (<>) {
  if (/^C\s+(\S+)\s+(\S+)\s+(\S+)\s*$/) {
    $x = $1;  $y = $2;   $r = $3;
    $_ = "# $_";
    for $i (0..$circ-1) {
      $v =($i + 0.5) * 8 * atan2(1,1) / $circ;
      $cx = $x + $r * cos($v);
      $cy = $y + $r * sin($v);
      $_ .= "P C$i $cx $cy\n";
    };
    $_ .= "L ";
    for $i (0..$circ-1) {$_ .= "C$i ";};
    $_ .= "C0\n";
  }
  $input .= $_;
};
for $_ (split(/\n/,$input)) {
  $error = 1;
  if      (/^P/) {
    if (/P\s+(\S+)\s+(\S+)\s+(\S+)\s*$/) {
      $x = $2*$sx+$tx-$cx;
      $y = $3*$sy+$ty-$cy;
      $xkoor{$1} = round($outx*(($cos*$x-$sin*$y+$cx)));
      $ykoor{$1} = round($outy*(($sin*$x+$cos*$y+$cy)));
      $error = 0;
    }
  } elsif (/^TR/) {
    if (/TR\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/) {
      $tx = $1;
      $ty = $2;
      $sx = $3;
      $sy = $4;
      $cx = $5 + $tx;
      $cy = $6 + $ty;
      $ang = ($7 / 45.0) * atan2(1,1);
      $sin = sin $ang;
      $cos = cos $ang;
      $error = 0;
    }
  } elsif (/^L/) {
    while (/^L\s+(\S+)\s+(\S+)\s*(.*)$/) {
      $output .=
      "Linie " .
      "$xkoor{$1} $ykoor{$1} $xkoor{$2} $ykoor{$2}\n";
      $error = 0;
      $_ = "L $2 $3";
    }
  } elsif (/^#/) {
    # Ignore comments
    $error = 0;
  } elsif (/^\s*$/) {
    # Ignore empty lines
    $error = 0;
  }
  #print STDERR "ERROR: $_\n" if ($error);
}

# transform coordinates into positive numbers

$file = "$before$output$after";
for $_ (split(/\n/,$file)) {
  if (/Linie\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s*$/) {
    $x1 = $1;
    $y1 = $2;
    $x2 = $3;
    $y2 = $4;
    $pixmap{$x1,$y1} = 1;
    $pixmap{$x2,$y2} = 1;
    if (abs($x2-$x1) >= abs($y2-$y1)) {
      if ($x1<$x2) {
        for $x ($x1..$x2) {
          $y = $y1+($y2-$y1)/($x2-$x1)*($x-$x1);
          $y = round($y);
          $pixmap{$x,$y} = 1;
        }
      } elsif ($x1==$x2) {
        if ($y1 > $y2) {
          for $y ($y2..$y1) {
            $pixmap{$x1,$y} = 1;
          }
        } else {
          for $y ($y1..$y2) {
            $pixmap{$x1,$y} = 1;
          }
        }
      } else {
        for $x ($x2..$x1) {
          $y = $y1+($y2-$y1)/($x2-$x1)*($x-$x1);
          $y = round($y);
          $pixmap{$x,$y} = 1;
        }
      }
    } else {
      if ($y1<$y2) {
        for $y ($y1..$y2) {
          $x = $x1+($x2-$x1)/($y2-$y1)*($y-$y1);
          $x = round($x);
          $pixmap{$x,$y} = 1;
        }
      } elsif ($y1==$y2) {
        if ($x1 > $x2) {
          for $x ($x2..$x1) {
            $pixmap{$x,$y1} = 1;
          }
        } else {
          for $x ($x1..$x2) {
            $pixmap{$x,$y1} = 1;
          }
        }
      } else {
        for $y ($y2..$y1) {
          $x = $x1+($x2-$x1)/($y2-$y1)*($y-$y1);
          $x = round($x);
          $pixmap{$x,$y} = 1;
        }
      }
    }

    $minx = $x1+0 unless defined $minx;
    $miny = $y1+0 unless defined $miny;
    $minx = $x1 if $x1 < $minx;
    $minx = $x2 if $x2 < $minx;
    $miny = $y1 if $y1 < $miny;
    $miny = $y2 if $y2 < $miny;
    $maxx = $x1+0 unless defined $maxx;
    $maxy = $y1+0 unless defined $maxy;
    $maxx = $x1 if $x1 > $maxx;
    $maxx = $x2 if $x2 > $maxx;
    $maxy = $y1 if $y1 > $maxy;
    $maxy = $y2 if $y2 > $maxy;
  }
}
print STDERR "Bounding box: X[$minx $maxx]  Y[$miny $maxy]\n";
$xpixs = $maxx - $minx + 1;
$ypixs = $maxy - $miny + 1;
print STDERR "generating a ppm with X\:$xpixs x Y\:$ypixs pixels\n";
# Output pixmap som ppm
print "P1 $xpixs $ypixs\n";
for $y (1..$ypixs) {
  $ry = $maxy + 1 - $y;
  for $x (1..$xpixs) {
    $rx = $minx + $x -1;
    if (defined $pixmap{$rx,$ry}) {
      print "0";
    } else {
      print "1";
    }
  }
  print "\n";
}
#
#  ------------------------------------------------------------------------
#October 21, 2001. Birger Nielsen, bnielsen@daimi.au.dk, drinker of tea.
#This document: http://hjem.get2net.dk/bnielsen/pl2ppm.html
