#nawk script generate multiple rotated copies of input $Revision: 1.9 $
#
#WBL  5 Dec 2005

#WBL  4 Jun 06 Make two pass, so bounding box is always correct. Add delay
#WBL 18 Apr 06 Support P3 too
#WBL  9 Dec 05 Supply random default for N, default seed set from FILENAME
#               Randomise delay

#Inputs P1/P3 graphics file
#       seed
#       N  number of outputs
#       outfile
#       delay (approx in 1/100 seconds between frames)
#       a = axis (x y z co-ordinates)
#       background colour for P3, pixels matching it are removed

BEGIN { 	
    PI = atan2(1,1)*4.0; 
    old=0;
    x=0;
    y=0;
    if(delay=="") delay=50;
    if(background=="") background="0 0 0"; #for P3
    if(a) {split(a,t); axis["x"]=t[1];axis["y"]=t[2];axis["z"]=t[3];}
#   print "axis",axis["x"],axis["y"],axis["z"];
}

(FNR > old  && index($0,"#")!=1)             { old=FNR;      nxtf=1;}
(type==""   && index($0,"#")!=1 && NF>=nxtf) { type  =$nxtf; nxtf++;}
(width==""  && index($0,"#")!=1 && NF>=nxtf) { width =$nxtf; nxtf++;}
(height=="" && index($0,"#")!=1 && NF>=nxtf) { height=$nxtf; nxtf++;
  if(type=="P1") header=FNR;
}
(maxval=="" && index($0,"#")!=1 && NF>=nxtf && type=="P3") {
    maxval=$nxtf; nxtf++;
    header=FNR;
}

(type=="P1" && header!="" && FNR>header) {
    for(i=1;i<=NF;i++) {
	for(j=1;j<=length($i);j++) {
	    if(substr($i,j,1)!="1") {
		pixmap[y,x]=substr($i,j,1); c++;
		set_bounds(x,y,pixmap);
	    }
	    x++;
	    if(x>=width) {
		x=0;
		y++;
	    }
	}
    }
}

(type=="P3" && header!="" && FNR>header) {
    for(i=1;i<=NF;i+=3) {
	p=sprintf("%s %s %s",$i,$(i+1),$(i+2));
	if(p!=background) {
	    pixmap[y,x]=p; c++;
	    set_bounds(x,y,pixmap);
	}
	x++;
	if(x>=width) {
	    x=0;
	    y++;
	}
    }
}

END {
#   print type,width,height,maxval,header, "'"background"'",c;
#   print "pixmap min/max",pixmap["minx"],pixmap["maxx"],pixmap["miny"],pixmap["maxy"];
    if(seed=="") {
	n=split(FILENAME,t,"[_.]");
	for(i=1;i<=n;i++) { if(isnum(t[i])) { seed=t[i]; break; }}
    }
    srand(seed);
    if(N=="") N=11+int(rand()*5);
#   print "rotate.awk $Revision: 1.9 $",FILENAME,seed,N,outfile>"/dev/stderr";

    if(height!=y) print "ERROR height(",height,") but ",y;
    gifmerge = "gifmerge -0,0,0 -l0 \\\n"; #set transparency, loop forever
    
    theta = 360.0/N*((rand()<0.5)? 1 : -1);
    angle = rand()*360;
#   angle = 0;
    if(axis["x"]=="") random_direction(axis);
    #first pass to set bounds for whole sequence
    Cx=(pixmap["minx"]+pixmap["maxx"])/2.0;
    Cy=(pixmap["miny"]+pixmap["maxy"])/2.0;
    for(i=0;i<N;i++) {
	glRotate(angle+i*theta,axis["x"],axis["y"],axis["z"],M);
	Rotatev(Cx,Cy,M,C);
 	Rotate(pixmap,M,newpix);
	ox[i]=newpix["minx"]-C[1];
	oy[i]=newpix["miny"]-C[2];
	mx=newpix["maxx"]-C[1];
	my=newpix["maxy"]-C[2];
	if(ox[i]<minx||minx=="") minx=ox[i];
	if(mx   >maxx||maxx=="") maxx=mx;
	if(oy[i]<miny||miny=="") miny=oy[i];
	if(my   >maxy||maxy=="") maxy=my;
#	print "newpix min/max",newpix["minx"],newpix["maxx"],newpix["miny"],newpix["maxy"],C[1]","C[2],"("ox[i],oy[i]")",minx,maxx,miny,maxy;
	for(j in newpix) delete newpix[j];
    }
    for(i=0;i<N;i++) {
	glRotate(angle+i*theta,axis["x"],axis["y"],axis["z"],M);
 	Rotate(pixmap,M,newpix);
 	printmap(newpix,delay,ox[i],oy[i],sprintf("%s%02d",outfile,i));
	for(j in newpix) delete newpix[j];
    }
#    system("sleep 2"); #give NFS time to settle
    #gifmerge is verbose, direct to file and check for errors later.
    gifmerge=sprintf("%s\t2> %s.err",gifmerge,outfile);
    gifmerge=sprintf("%s\t> %s.gif",gifmerge,outfile);
#   print gifmerge;
    system(gifmerge);
    s=sprintf("egrep -v '(^=== GIFMerge|^    Improvements|^Transparency:|^Loops:|^Position:|^Merging|^Delay:)' %s.err",outfile);
#   s=sprintf("cat %s.err",outfile);
    system(s);
}

function isnum(x) {
  return (0+x)==x;
}
function max(a,b) { return (a>b)? a : b; }
function default() {return (type=="P3")? "0 0 0" : "1"; }
function printmap(pixmap,delay,ox,oy,file, w,h,px,py,J,I,j,i) {
    w=round(maxx-minx+1);
    h=round(maxy-miny+1);
    py=-(miny-oy);
    px=-(minx-ox);
    J=(py<=0)? pixmap["miny"] : round(pixmap["miny"]-py);
    I=(px<=0)? pixmap["minx"] : round(pixmap["minx"]-px);
#   print "printmap(pixmap",delay,ox,oy, file ",)", w"x"h, "("px,py")", I,J;
    gifmerge = sprintf("%s\t-%d %s.gif \\\n",
	       gifmerge,delay,file);
    file=sprintf("ppmtogif 2>/dev/null >%s.gif",file)
#   file=sprintf("cat >%s.ppm",file)
    print type, w, h, maxval | file;
    for(j=J;j<J+h;j++) {
    for(i=I;i<I+w;i++) {
	    if(pixmap[j,i]) printf("%s",pixmap[j,i]) | file;
	    else            printf(default())        | file;
	    if(type=="P3") printf("\n")              | file;
	}
	if(type!="P3") printf("\n")                  | file;
    }
    close(file);
}
function round(x) { return int(x+0.5); }
function Rotate(pixmap,M,newpix, y,m01y,m11y,x,x_,y_) {/*bitmap_output*/
							    
  //noneed to center 

  for(y=pixmap["miny"];y<=pixmap["maxy"];y++) {
    m01y = M[0,1]*y;
    m11y = M[1,1]*y;
    for(x=pixmap["minx"];x<=pixmap["maxx"];x++) {
	if(pixmap[y,x]) {
      x_ = round(M[0,0]*x + m01y);
      y_ = round(M[1,0]*x + m11y);
      set_bounds(x_,y_,newpix);
      newpix[y_,x_] = pixmap[y,x];
	}
    }
  }  
}
function Rotatev(x,y,M,Out) {
    Out[1] = (M[0,0]*x + M[0,1]*y);
    Out[2] = (M[1,0]*x + M[1,1]*y);
}
# WBL 5 Dec 2005 from lsys.js Revision: 1.26 

#from random_direction.js Revision: 1.2
function random_direction(axis, w2) {
  do{#scatter randomly across a positive half cube
     #but then reject values close to origin or outside unit sphere
  axis["x"] = rand()*2-1;
  axis["y"] = rand()*2-1;
  axis["z"] = rand();
  w2 = axis["x"]*axis["x"] + axis["y"]*axis["y"] + axis["z"]*axis["z"];
  } while(w2<0.01||w2>1)
}
function set_bounds(x,y,pixmap) {
  if(pixmap["minx"]==""|| x < pixmap["minx"]) {pixmap["minx"]=x;}
  if(pixmap["maxx"]==""|| x > pixmap["maxx"]) {pixmap["maxx"]=x;}
  if(pixmap["miny"]==""|| y < pixmap["miny"]) {pixmap["miny"]=y;}
  if(pixmap["maxy"]==""|| y > pixmap["maxy"]) {pixmap["maxy"]=y;}
}


#from glRotate.js Revision: 1.1 
function glRotate(angle,x,y,z,M, x2,y2,z2,w2,w,c,s) {
# print "glRotate(",angle,x,y,z,"M)";
  #Normalise x,y,z
  x2 = x*x;
  y2 = y*y;
  z2 = z*z;
  w2 = x*x + y*y + z*z;
  if(w2!=1) {
    x2 /= w2;
    y2 /= w2;
    z2 /= w2;
    w=sqrt(w2);
    x  /= w;
    y  /= w;
    z  /= w;
  }
  angle *= PI/180;
  c = cos(angle); cc = 1-c;
  s = sin(angle);

# M = new Array(); 
# M[0] = new Array(); M[1]= new Array();  M[2]= new Array();    M[3]=new Array;

  M[0,0]= x2*cc+c;   M[0,1]=x*y*cc-z*s; M[0,2]=x*z*cc+y*s; #M[0,3]
  M[1,0]=x*y*cc+z*s; M[1,1]= y2*cc+c;   M[1,2]=y*z*cc-x*s; #M[1,3]
  M[2,0]=x*z*cc-y*s; M[2,1]=y*z*cc+x*s; M[2,2]= z2*cc+c;   #M[2,3]
# M[3,0]=          ; M[3,1]=;           M[3,2]=;*/          M[3,3]=1;

#  print angle,M[0,0],M[0,1],M[1,0],M[1,1] >"/dev/stderr";

#  return M;
}

