// return one or more random lines of a file via CGI
//
// Based http://cscene.org/CS6/CS6-07.html 19 Dec 2001
// W. B. Langdon

//WBL 25 Jun 2005 Make record number 10 chars wide and use | to separate seeds
//WBL 17 Jun 2005 Add record number
//WBL 29 Dec 2001 Reduce logging but add ip 
//WBL 28 Dec 2001 Add removal of date/time/ip header

// compile with g++ -o seed3 seed3.cc
//
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <time.h>
#include <vector>

int gpquick_seed = 192754;
#include "../gp/comb/pch.h"

#define strcasecmp strcmp

#if _WIN32
#include <fcntl.h>
#include <io.h>
#define strcasecmp stricmp
#endif

ofstream debug ("debug",ios::app);

void cgiError()
{
    // Most simple error handling
    cout << endl;
    exit(0);
}

void dump(const char* n, const char* buf, const char* sep)
{
  debug<<" dumpFile("<<n<<buf<<")" <<flush;

  cout<<n<<buf<<sep<<flush;
}

char* check_header(/*const*/ char* buf, int& size) {
  //const char* Alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  const char* alphabet = "abcdefghijklmnopqrstuvwxyz";
  //  const char* digits   = "0123456789";
  char* p;
  if(size>23 &&
     strspn(&buf[0],"SMTWF")==1 &&
     strspn(&buf[1],alphabet)==2 &&
     strspn(&buf[4],"JFMASOND")==1 &&
     strspn(&buf[5],alphabet)==2 &&
     strspn(&buf[7]," :0123456789")>15 &&
     (p = strchr(&buf[22],'|')) != NULL) {
    p++;
    size -= (p-buf);
    return p;
  }
  return buf;
}

int main()
{
    cout << "Content-Type: text/plain\n\n";

    debug<<"seed main()"<<flush;
    time_t start;
    time (&start);
    char* timestr = ctime(&start); timestr[strlen(timestr)-1]='\0';
    debug << timestr << " ";
    debug << getenv("REMOTE_HOST") << " ";
    debug << getenv("REMOTE_ADDR") << "|"; /* << flush;*/
#if _WIN32
    // Standard I/O is in text mode by default; since we intend
    // to send binary image data to standard output, we have to
    // set it to binary mode.
    // Error handling is tricky to say the least, so we have none.
    _fmode = _O_BINARY;
    if (_setmode(_fileno(stdin), _fmode) == -1) {}
    if (_setmode(_fileno(stdout), _fmode) == -1) {}
#endif

    // First, we get the text to send back
    char* querystr = getenv("QUERY_STRING");
    debug<<" '"<<querystr<<flush;
    const int nseeds = (querystr || querystr[0]!='\0') ? atoi(querystr) : 1;
    const char* r = strchr(querystr,'_');
    const int rep = (r==NULL)? 0 : atoi(r+1);
    debug<<"' ("<<nseeds<<" "<<rep<<") "<<flush;

    ifstream seedf("seed.pop");
    if(seedf) /*debug<<"opened seed.pop "<<flush*/;
    else {
      debug<<"failed to open seed.pop "<<flush;
      cout <<endl;
      exit(0);
    }

    vector<char*> seeds; 
    int seed_count = 0;

    char buf[32768];
    char c;
    //debug<<" Reading seed.pop "<<flush;
    do {
      char* p = buf;
      for(c=seedf.get(); c!='\n' && c!=EOF; p++) { 
	//debug<<" r"<<int(c)<<flush;
	*p = c; 
	c = seedf.get();
      }
      *p++='\0';
      //debug<<" read "<<int(c)<<"'"<<buf<<"'"<<flush;
      int size = p-buf;
      //debug<<size<<" "<<flush;
      if(size>1) {
	p = check_header(buf,size);
	char* t = new char[size];
	memcpy(t,p,size); 
	seeds.insert(seeds.begin()+seed_count, t); 
	seed_count++;
      }
    } while(c!=EOF);
    debug<<"read "<<seed_count<<" records from seed.pop "<<flush;

    rndomize(); {for(int i=0;i<rep;i++) rnd(seed_count);}//discard rep numbers
    for(int i=0;i<nseeds;i++) {
      const int r = rnd(seed_count);
      char rs[11];
      sprintf(rs,"%010d",r);
      debug<<" sending["<<r<<"] "<<flush;
      //debug<<seeds[r]<<flush;
      dump(rs, seeds[r], (i<nseeds-1)? "|":"");
    }

    cout << endl;

    debug<<"seed3 exit"<<endl;
    return 0;
}

